import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Separator fr log4j-Log-Files nach den unterschiedlichen
 * Threads.
 * 
 * Damit dieses Programm funktioniert, muss folgendes
 * log4j-Pattern vewrwendet werden
 * log4j.appender.logfile.stdout.ConversionPattern=%5p %d{ISO8601} [%t] %l - %m%n
 * log4j.appender.logfile.layout.ConversionPattern=%5p %d{ISO8601} [%t] %l - %m%n
 * 
 * Mit diesem Pattern klappt auch das An-Navigieren der Codestellen
 * aus der Eclipse-Console.
 * 
 * Das angegeben log4j-Lof-File wird eingelesen
 * und die Zeilen nach den einzelnen Threads 
 * unterteilt.
 * 
 * Eine Thread-Zeile kann folgendermassen aussehen:
 * 
 * DEBUG 2006-04-10 15:57:50,924 [Thread 1] (Example.java:100) - message
 * 
 * 
 * Beim Auftreten einer Exception oder einer mehrzeiligen Ausgabe
 * werden die Folgezeilen dem zuletzt festgestellten Thread
 * zugeordnet:
 * 
 * ERROR 2006-04-10 11:23:28,602 [Thread 99] (Example.java:281) - 
 * java.net.ConnectException: Connection refused: connect
 *   at java.net.PlainSocketImpl.socketConnect(Native Method)
 *   at java.net.PlainSocketImpl.doConnect(Unknown Source)
 * 
 * ber die Konstante {@link #DEFAULT_LOG_FILE_NAME}
 * kann ein Standard-Log-File-Name vergeben werden.
 * 
 * ber die Konstante {@link #NOT_TO_WRITE_THREAD_PREFIX_ARR}
 * knnen bestimmte Threads unterdrckt werden.
 * 
 * Die Ausgabe der einzelnen Threads erfolgt in Dateien mit 
 * dem Namen &lt;thread-name (ohne sonderzeichen)&gt;_&lt;log-file-name&gt;.
 * 
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner kcker
 */
public class LogFileThreadSeparator
{
  /**
   * Diese Log-Datei wird verwendet,
   * wenn die main-Methode keinen 
   * Parameter bekommen hat.
   */
  public static final String DEFAULT_LOG_FILE_NAME = "C:\\Workspace_Eclipse\\application\\application.log";

  /**
   * Array mit den Anfngen (Prefix) der Threads,
   * die nicht ausgegeben werden sollen
   */
  public static final String[] NOT_TO_WRITE_THREAD_PREFIX_ARR = { };
  
  /**
   * die Text Zeilen fr die jeweiligen Threads werden in dieser Map gespeichert
   */
  private static final HashMap<String, ArrayList<String>> threadLinesMap = //--
    new HashMap<String, ArrayList<String>>();

  /**
   * Main.
   * @param args
   */
  public static void main(String[] args)
  {
    if (args == null || args.length != 1)
    {
      System.out.println( "Aufruf mit <source-file>" );
      //System.exit(1);
    }
    
    final String srcFileName;
    if (args == null || args.length > 0 )
    {
      srcFileName = args[0];
    }
    else
    {
      //System.out.print( "Source log file:" );
      //srcFileName = readConsole();
      srcFileName = DEFAULT_LOG_FILE_NAME;
    }

    System.out.println( "reading source-file: " + srcFileName );

    //Log-File einlesen
    final File srcFile = new File( srcFileName );
    String[] lineArr = StringUtil.file2StrArr( srcFile );
    final int srcLineCnt = lineArr.length;
    
    String threadName = null;
    for (int i = 0; i < srcLineCnt; i++)
    {
      final String lineStr = lineArr[i];

      if ( isThreadLine( lineStr ) )
      {
        // es ist die erste Zeile fr diesen Thread
        // Thread-Name vermerken
        threadName = readThreadName( lineStr );
      }
      if ( threadName != null )
      {
        //Log-File-Zeile fr den entsprechenden Thread vermerken
        addLine( threadName ,  lineStr );
      }
    }
    
    //Schreibe in Zieldateien
    for (Iterator iter = threadLinesMap.keySet().iterator(); iter.hasNext();)
    {
      threadName = (String) iter.next();
      final String threadNameCleaned = StringUtil.onlyDigitsOrUsLetters( threadName );
      if ( isNotToWriteThread( threadNameCleaned ) )
        // diese Threads nicht dumpen
      {
        continue;
      }
      
      final ArrayList<String> lineList = /*(ArrayList<String>)*/ threadLinesMap.get( threadName );
      
      lineArr = /*(String[])*/ lineList.toArray( new String[ lineList.size() ] );

      final String dstFileName = srcFile.getParent() + File.separator + threadNameCleaned + "_" + srcFile.getName();
      
      System.out.println( "write " + lineArr.length + " lines to " + dstFileName );
      StringUtil.strArr2file( lineArr , dstFileName );
    }
  }

  /**
   * ob dieser Thread nicht auf eine Datei
   * ausgeben werden soll.
   * 
   * @param pThreadNameCleaned
   * @return
   */
  private static boolean isNotToWriteThread(
      final String pThreadNameCleaned )
  {
    for (int i = 0; i < NOT_TO_WRITE_THREAD_PREFIX_ARR.length; i++ )
    {
      final String notToWriteThreadPrefixStr = NOT_TO_WRITE_THREAD_PREFIX_ARR[i];
      if ( pThreadNameCleaned.startsWith( notToWriteThreadPrefixStr ) )
      {
        return true;
      }
    }
    return false;
  }

  /**
   * Prfen, ob die bergebene Zeile die
   * erste fr eine bestimmten Thread ist
   * oder eine Folgezeile, wie sie zum
   * Beispiel bei Exception StackTraces
   * ausgegeben wird, ist.
   * 
   * @param pLineStr
   * @return ob erste Thread-Zeile
   */
  private static boolean isThreadLine(
      final String pLineStr )
  {
    //Pattern
    //012345678901234567890123456789012345678901234567890123456789
    //DEBUG 2006-03-17 22:08:19,381 [main] (Main:134) - Start Application
    if (( pLineStr.startsWith( "DEBUG 20" ) //--
          || pLineStr.startsWith( " INFO 20" ) //--
          || pLineStr.startsWith( "ERROR 20" ) //--
          || pLineStr.startsWith( "FATAL 20" ) ) //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 8 ) ) //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 9 ) ) //--
        && StringUtil.charAt( pLineStr , 10 ) == '-' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 11 ) ) //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 12 ) ) //--
        && StringUtil.charAt( pLineStr , 13 ) == '-' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 14 ) ) //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 15 ) ) //--
        && StringUtil.charAt( pLineStr , 16 ) == ' ' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 17 ) )//--
        && Character.isDigit( StringUtil.charAt( pLineStr , 18 ) )//--
        && StringUtil.charAt( pLineStr , 19 ) == ':' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 20 ) )//--
        && Character.isDigit( StringUtil.charAt( pLineStr , 21 ) )//--
        && StringUtil.charAt( pLineStr , 22 ) == ':' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 23 ) )//--
        && Character.isDigit( StringUtil.charAt( pLineStr , 24 ) )//--
        && StringUtil.charAt( pLineStr , 25 ) == ',' //--
        && Character.isDigit( StringUtil.charAt( pLineStr , 26 ) )//--
        && Character.isDigit( StringUtil.charAt( pLineStr , 27 ) )//--
        && Character.isDigit( StringUtil.charAt( pLineStr , 28 ) )//--
        && StringUtil.charAt( pLineStr , 29 ) == ' ' //--
        && StringUtil.charAt( pLineStr , 30 ) == '[' //--
    )
    {
      return true;
    }
    return false;
  }

  /**
   * @param pLineStr
   * @return
   */
  private static String readThreadName(
      final String pLineStr )
  {
    final String rightStr = pLineStr.substring( 31 );
    return StringUtil.leftUpToTheChr( rightStr , ']' );
  }

  /**
   * add log file text line to the specified thread.
   * @param pThreadName
   * @param pLine
   */
  private static void addLine( 
      final String pThreadName, //--
      final String pLine)
  {
    ArrayList<String> lineList = threadLinesMap.get( pThreadName );
    if ( lineList == null )
    {
      lineList = new ArrayList<String>();
      threadLinesMap.put( pThreadName, lineList );
    }
    lineList.add( pLine );
  }
  
  /**
   * Read string from console.
   * 
   * @return
   */
  private static String readConsole()
  {
    InputStreamReader isr = new InputStreamReader(System.in);
    BufferedReader input = new BufferedReader(isr);
    
    String line = null;
    
    try
    {
      line = input.readLine();
    }
    catch (IOException e)
    {
      System.err.println(e.getMessage());
      e.printStackTrace();
    }
    return line;
  }
  
}
